@@ -17,8 +17,6 @@ Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-@bug When switching to tab while logging in, it doesn't refresh once logged int
 """
 
 
@@ -113,7 +111,6 @@ class Dialcentral(object):
 		self._window = hildonize.hildonize_window(self._app, self._window)
 		hildonize.hildonize_text_entry(self._widgetTree.get_widget("usernameentry"))
 		hildonize.hildonize_password_entry(self._widgetTree.get_widget("passwordentry"))
-		hildonize.hildonize_combo_entry(self._widgetTree.get_widget("callbackcombo").get_child())
 
 		for scrollingWidget in (
 			'recent_scrolledwindow',
@@ -136,10 +133,9 @@ class Dialcentral(object):
 			replacementButtons
 		)
 
-		if hildonize.IS_HILDON_SUPPORTED:
-			self._window.connect("key-press-event", self._on_key_press)
-			self._window.connect("window-state-event", self._on_window_state_change)
-		else:
+		self._window.connect("key-press-event", self._on_key_press)
+		self._window.connect("window-state-event", self._on_window_state_change)
+		if not hildonize.IS_HILDON_SUPPORTED:
 			logging.warning("No hildonization support")
 
 		hildonize.set_application_title(self._window, "%s" % constants.__pretty_app_name__)
@@ -456,11 +452,12 @@ class Dialcentral(object):
 
 		if self._phoneBackends[self._selectedBackendId].get_callback_number() is None:
 			self._phoneBackends[self._selectedBackendId].set_sane_callback()
-		self._accountViews[self._selectedBackendId].update()
-		self._refresh_active_tab()
 
 		self._selectedBackendId = newStatus
 
+		self._accountViews[self._selectedBackendId].update()
+		self._refresh_active_tab()
+
 	def load_settings(self, config):
 		"""
 		@note UI Thread
@@ -655,7 +652,10 @@ class Dialcentral(object):
 		@note Hildon specific
 		"""
 		try:
-			if event.keyval == gtk.keysyms.F6:
+			if (
+				event.keyval == gtk.keysyms.F6 or
+				event.keyval == gtk.keysyms.Return and event.get_state() & gtk.gdk.CONTROL_MASK
+			):
 				if self._isFullScreen:
 					self._window.unfullscreen()
 				else:
