@@ -18,10 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-@todo Touch selector for callback number
 @todo Alternate UI for dialogs (stackables)
-@todo Switch to a selector with entry for notification time
-@bug Messages can't refresh
 """
 
 from __future__ import with_statement
@@ -671,10 +668,10 @@ class AccountInfo(object):
 		self._notifyOnVoicemail = False
 		self._notifyOnSms = False
 
-		self._callbackList = gtk.ListStore(gobject.TYPE_STRING)
+		self._callbackList = []
 		self._accountViewNumberDisplay = widgetTree.get_widget("gcnumber_display")
-		self._callbackCombo = widgetTree.get_widget("callbackcombo")
-		self._onCallbackentryChangedId = 0
+		self._callbackSelectButton = widgetTree.get_widget("callbackSelectButton")
+		self._onCallbackSelectChangedId = 0
 
 		self._notifyCheckbox = widgetTree.get_widget("notifyCheckbox")
 		self._minutesEntryButton = widgetTree.get_widget("minutesEntryButton")
@@ -697,8 +694,8 @@ class AccountInfo(object):
 		self._accountViewNumberDisplay.set_use_markup(True)
 		self.set_account_number("")
 
-		self._callbackList.clear()
-		self._onCallbackentryChangedId = self._callbackCombo.get_child().connect("changed", self._on_callbackentry_changed)
+		del self._callbackList[:]
+		self._onCallbackSelectChangedId = self._callbackSelectButton.connect("clicked", self._on_callbackentry_clicked)
 
 		if self._alarmHandler is not None:
 			self._notifyCheckbox.set_active(self._alarmHandler.isEnabled)
@@ -722,8 +719,8 @@ class AccountInfo(object):
 		self.update(force=True)
 
 	def disable(self):
-		self._callbackCombo.get_child().disconnect(self._onCallbackentryChangedId)
-		self._onCallbackentryChangedId = 0
+		self._callbackSelectButton.disconnect(self._onCallbackSelectChangedId)
+		self._onCallbackSelectChangedId = 0
 
 		if self._alarmHandler is not None:
 			self._notifyCheckbox.disconnect(self._onNotifyToggled)
@@ -744,10 +741,14 @@ class AccountInfo(object):
 			self._smsCheckbox.set_sensitive(True)
 
 		self.clear()
-		self._callbackList.clear()
+		del self._callbackList[:]
 
 	def get_selected_callback_number(self):
-		return make_ugly(self._callbackCombo.get_child().get_text())
+		currentLabel = self._callbackSelectButton.get_label()
+		if currentLabel is not None:
+			return make_ugly(currentLabel)
+		else:
+			return ""
 
 	def set_account_number(self, number):
 		"""
@@ -763,7 +764,7 @@ class AccountInfo(object):
 		return True
 
 	def clear(self):
-		self._callbackCombo.get_child().set_text("")
+		self._callbackSelectButton.set_label("")
 		self.set_account_number("")
 		self._isPopulated = False
 
@@ -792,7 +793,7 @@ class AccountInfo(object):
 
 	def _populate_callback_combo(self):
 		self._isPopulated = True
-		self._callbackList.clear()
+		del self._callbackList[:]
 		try:
 			callbackNumbers = self._backend.get_callback_numbers()
 		except Exception, e:
@@ -801,13 +802,10 @@ class AccountInfo(object):
 			return
 
 		for number, description in callbackNumbers.iteritems():
-			self._callbackList.append((make_pretty(number),))
+			self._callbackList.append(make_pretty(number))
 
-		self._callbackCombo.set_model(self._callbackList)
-		self._callbackCombo.set_text_column(0)
-		#callbackNumber = self._backend.get_callback_number()
-		callbackNumber = self._defaultCallback
-		self._callbackCombo.get_child().set_text(make_pretty(callbackNumber))
+		if not self.get_selected_callback_number():
+			self._set_callback_number(self._defaultCallback)
 
 	def _set_callback_number(self, number):
 		try:
@@ -824,6 +822,7 @@ class AccountInfo(object):
 				assert make_ugly(number) == make_ugly(self._backend.get_callback_number()), "Callback number should be %s but instead is %s" % (
 					make_pretty(number), make_pretty(self._backend.get_callback_number())
 				)
+				self._callbackSelectButton.set_label(make_pretty(number))
 				logging.info(
 					"Callback number set to %s" % (
 						self._backend.get_callback_number(),
@@ -842,11 +841,20 @@ class AccountInfo(object):
 			self._notifyCheckbox.set_active(self._alarmHandler.isEnabled)
 			self._minutesEntryButton.set_label("%d Minutes" % self._alarmHandler.recurrence)
 
-	def _on_callbackentry_changed(self, *args):
+	def _on_callbackentry_clicked(self, *args):
 		try:
-			text = self.get_selected_callback_number()
-			number = make_ugly(text)
+			actualSelection = make_pretty(self.get_selected_callback_number())
+
+			userSelection = hildonize.touch_selector_entry(
+				self._window,
+				"Callback Number",
+				self._callbackList,
+				actualSelection,
+			)
+			number = make_ugly(userSelection)
 			self._set_callback_number(number)
+		except RuntimeError, e:
+			logging.exception("%s" % str(e))
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -1142,6 +1150,7 @@ class MessagesView(object):
 	def enable(self):
 		assert self._backend.is_authed(), "Attempting to enable backend while not logged in"
 		self._messageview.set_model(self._messagemodel)
+		self._messageview.set_headers_visible(False)
 
 		self._messageview.append_column(self._messageColumn)
 		self._messageviewselection = self._messageview.get_selection()
